@@ -58,8 +58,13 @@ typedef VkBool32 (APIENTRY *PFN_vkGetPhysicalDeviceWaylandPresentationSupportKHR
 #include "wayland-pointer-constraints-unstable-v1-client-protocol.h"
 #include "wayland-primary-selection-unstable-v1-client-protocol.h"
 #include "wayland-primary-selection-unstable-v1-client-protocol.h"
-#include "wl_text_input.h"
 #include "wayland-xdg-activation-v1-client-protocol.h"
+#include "wayland-cursor-shape-v1-client-protocol.h"
+#include "wayland-fractional-scale-v1-client-protocol.h"
+#include "wayland-viewporter-client-protocol.h"
+#include "wayland-kwin-blur-v1-client-protocol.h"
+#include "wayland-wlr-layer-shell-unstable-v1-client-protocol.h"
+#include "wayland-single-pixel-buffer-v1-client-protocol.h"
 
 #define _glfw_dlopen(name) dlopen(name, RTLD_LAZY | RTLD_LOCAL)
 #define _glfw_dlclose(handle) dlclose(handle)
@@ -89,29 +94,27 @@ typedef void (* PFN_wl_egl_window_resize)(struct wl_egl_window*, int, int, int,
 #define wl_egl_window_destroy _glfw.wl.egl.window_destroy
 #define wl_egl_window_resize _glfw.wl.egl.window_resize
 
-typedef enum _GLFWdecorationSideWayland
+typedef enum _GLFWCSDSurface
 {
-    CENTRAL_WINDOW,
-    TOP_DECORATION,
-    LEFT_DECORATION,
-    RIGHT_DECORATION,
-    BOTTOM_DECORATION,
-} _GLFWdecorationSideWayland;
+    CENTRAL_WINDOW, CSD_titlebar, CSD_shadow_top, CSD_shadow_left, CSD_shadow_bottom, CSD_shadow_right,
+    CSD_shadow_upper_left, CSD_shadow_upper_right, CSD_shadow_lower_left, CSD_shadow_lower_right,
+} _GLFWCSDSurface;
 
 typedef struct _GLFWWaylandBufferPair {
     struct wl_buffer *a, *b, *front, *back;
     struct { uint8_t *a, *b, *front, *back; } data;
     bool has_pending_update;
-    size_t size_in_bytes, width, height, stride;
+    size_t size_in_bytes, width, height, viewport_width, viewport_height, stride;
     bool a_needs_to_be_destroyed, b_needs_to_be_destroyed;
 } _GLFWWaylandBufferPair;
 
-typedef struct _GLFWWaylandCSDEdge {
+typedef struct _GLFWWaylandCSDSurface {
     struct wl_surface *surface;
     struct wl_subsurface *subsurface;
+    struct wp_viewport *wp_viewport;
     _GLFWWaylandBufferPair buffer;
     int x, y;
-} _GLFWWaylandCSDEdge;
+} _GLFWWaylandCSDSurface;
 
 typedef enum WaylandWindowState {
 
@@ -143,6 +146,13 @@ enum WaylandWindowPendingState {
     PENDING_STATE_DECORATION = 2
 };
 
+enum _GLFWWaylandAxisEvent {
+    AXIS_EVENT_UNKNOWN = 0,
+    AXIS_EVENT_CONTINUOUS = 1,
+    AXIS_EVENT_DISCRETE = 2,
+    AXIS_EVENT_VALUE120 = 3
+};
+
 // Wayland-specific per-window data
 //
 typedef struct _GLFWwindowWayland
@@ -160,7 +170,34 @@ typedef struct _GLFWwindowWayland
         struct xdg_surface*     surface;
         struct xdg_toplevel*    toplevel;
         struct zxdg_toplevel_decoration_v1* decoration;
+        struct { int width, height; } top_level_bounds;
     } xdg;
+    struct wp_fractional_scale_v1 *wp_fractional_scale_v1;
+    struct wp_viewport *wp_viewport;
+    struct org_kde_kwin_blur *org_kde_kwin_blur;
+    bool has_blur, expect_scale_from_compositor, window_fully_created;
+    struct {
+        bool surface_configured, fractional_scale_received, preferred_scale_received;
+    } once;
+    struct wl_buffer *temp_buffer_used_during_window_creation;
+    struct {
+        GLFWLayerShellConfig config;
+        struct zwlr_layer_surface_v1* zwlr_layer_surface_v1;
+    } layer_shell;
+
+    /* information about axis events on current frame */
+    struct
+    {
+        struct {
+            enum _GLFWWaylandAxisEvent x_axis_type;
+            float x;
+            enum _GLFWWaylandAxisEvent y_axis_type;
+            float y;
+        } discrete, continuous;
+
+        /* Event timestamp in nanoseconds */
+        monotonic_t timestamp_ns;
+    } pointer_curr_axis_info;
 
     _GLFWcursor*                currentCursor;
     double                      cursorPosX, cursorPosY, allCursorPosX, allCursorPosY;
@@ -170,7 +207,8 @@ typedef struct _GLFWwindowWayland
 
     // We need to track the monitors the window spans on to calculate the
     // optimal scaling factor.
-    int                         scale;
+    struct { uint32_t deduced, preferred; } integer_scale;
+    uint32_t                    fractional_scale;
     bool                        initial_scale_notified;
     _GLFWmonitor**              monitors;
     int                         monitorsCount;
@@ -182,9 +220,10 @@ typedef struct _GLFWwindowWayland
     } pointerLock;
 
     struct {
-        bool serverSide, buffer_destroyed;
-        _GLFWdecorationSideWayland focus;
-        _GLFWWaylandCSDEdge top, left, right, bottom;
+        bool serverSide, buffer_destroyed, titlebar_needs_update;
+        _GLFWCSDSurface focus;
+
+        _GLFWWaylandCSDSurface titlebar, shadow_left, shadow_right, shadow_top, shadow_bottom, shadow_upper_left, shadow_upper_right, shadow_lower_left, shadow_lower_right;
 
         struct {
             uint8_t *data;
@@ -192,8 +231,10 @@ typedef struct _GLFWwindowWayland
         } mapping;
 
         struct {
-            int width, height, scale;
-            bool focused;
+            int width, height;
+            bool focused, needs_shadow;
+            float fscale;
+            WaylandWindowState toplevel_states;
         } for_window_state;
 
         struct {
@@ -204,6 +245,11 @@ typedef struct _GLFWwindowWayland
             int32_t x, y, width, height;
         } geometry;
 
+        struct {
+            bool hovered;
+            int width, left;
+        } minimize, maximize, close;
+
         struct {
             uint32_t *data;
             size_t for_decoration_size, stride, segments, corner_size;
@@ -224,18 +270,16 @@ typedef struct _GLFWwindowWayland
         int32_t width, height;
     } user_requested_content_size;
 
-    bool maximize_on_first_show;
-    // counters for ignoring axis events following axis_discrete events in the
-    // same frame along the same axis
     struct {
-        unsigned int x, y;
-    } axis_discrete_count;
-    bool surface_configured_once;
+        bool minimize, maximize, fullscreen, window_menu;
+    } wm_capabilities;
 
+
+    bool maximize_on_first_show;
     uint32_t pending_state;
     struct {
         int width, height;
-        uint32_t toplevel_states;
+        WaylandWindowState toplevel_states;
         uint32_t decoration_mode;
     } current, pending;
 } _GLFWwindowWayland;
@@ -278,6 +322,7 @@ typedef struct _GLFWlibraryWayland
     struct wl_data_device_manager*          dataDeviceManager;
     struct wl_data_device*      dataDevice;
     struct xdg_wm_base*         wmBase;
+    int xdg_wm_base_version;
     struct zxdg_decoration_manager_v1*      decorationManager;
     struct zwp_relative_pointer_manager_v1* relativePointerManager;
     struct zwp_pointer_constraints_v1*      pointerConstraints;
@@ -286,13 +331,20 @@ typedef struct _GLFWlibraryWayland
     struct zwp_primary_selection_device_v1*    primarySelectionDevice;
     struct zwp_primary_selection_source_v1*    dataSourceForPrimarySelection;
     struct xdg_activation_v1* xdg_activation_v1;
+    struct wp_cursor_shape_manager_v1* wp_cursor_shape_manager_v1;
+    struct wp_cursor_shape_device_v1* wp_cursor_shape_device_v1;
+    struct wp_fractional_scale_manager_v1 *wp_fractional_scale_manager_v1;
+    struct wp_viewporter *wp_viewporter;
+    struct org_kde_kwin_blur_manager *org_kde_kwin_blur_manager;
+    struct zwlr_layer_shell_v1* zwlr_layer_shell_v1; uint32_t zwlr_layer_shell_v1_version;
+    struct wp_single_pixel_buffer_manager_v1 *wp_single_pixel_buffer_manager_v1;
 
     int                         compositorVersion;
     int                         seatVersion;
 
     struct wl_surface*          cursorSurface;
     GLFWCursorShape             cursorPreviousShape;
-    uint32_t                    serial, input_serial;
+    uint32_t                    serial, input_serial, pointer_serial, pointer_enter_serial, keyboard_enter_serial;
 
     int32_t                     keyboardRepeatRate;
     monotonic_t                 keyboardRepeatDelay;
@@ -334,6 +386,7 @@ typedef struct _GLFWlibraryWayland
     EventLoopData eventLoopData;
     size_t dataOffersCounter;
     _GLFWWaylandDataOffer dataOffers[8];
+    bool has_preferred_buffer_scale;
 } _GLFWlibraryWayland;
 
 // Wayland-specific per-monitor data
@@ -370,6 +423,15 @@ void _glfwAddOutputWayland(uint32_t name, uint32_t version);
 void _glfwWaylandAfterBufferSwap(_GLFWwindow *window);
 void _glfwSetupWaylandDataDevice(void);
 void _glfwSetupWaylandPrimarySelectionDevice(void);
+float _glfwWaylandWindowScale(_GLFWwindow*);
+int _glfwWaylandIntegerWindowScale(_GLFWwindow*);
 void animateCursorImage(id_type timer_id, void *data);
 struct wl_cursor* _glfwLoadCursor(GLFWCursorShape, struct wl_cursor_theme*);
 void destroy_data_offer(_GLFWWaylandDataOffer*);
+
+typedef struct wayland_cursor_shape {
+    int which; const char *name;
+} wayland_cursor_shape;
+
+wayland_cursor_shape
+glfw_cursor_shape_to_wayland_cursor_shape(GLFWCursorShape g);
