@@ -1,21 +1,26 @@
-#!/usr/bin/env python3
+#!/usr/bin/env python
 # License: GPL v3 Copyright: 2018, Kovid Goyal <kovid at kovidgoyal.net>
 
 
 from collections import deque
 from contextlib import suppress
-from time import monotonic
 from types import TracebackType
-from typing import (
-    TYPE_CHECKING, Any, Callable, ContextManager, Deque, Dict, NamedTuple,
-    Optional, Sequence, Type, Union, cast
-)
+from typing import TYPE_CHECKING, Any, Callable, ContextManager, Deque, Dict, NamedTuple, Optional, Sequence, Type, Union, cast
 
+from kitty.fast_data_types import monotonic
 from kitty.types import DecoratedFunc, ParsedShortcut
 from kitty.typing import (
-    AbstractEventLoop, BossType, Debug, ImageManagerType, KeyActionType,
-    KeyEventType, LoopType, MouseButton, MouseEvent, ScreenSize,
-    TermManagerType
+    AbstractEventLoop,
+    BossType,
+    Debug,
+    ImageManagerType,
+    KeyActionType,
+    KeyEventType,
+    LoopType,
+    MouseButton,
+    MouseEvent,
+    ScreenSize,
+    TermManagerType,
 )
 
 from .operations import MouseTracking, pending_update
