@@ -27,7 +27,7 @@ extern PyTypeObject Screen_Type;
 #endif
 
 #ifdef DEBUG_EVENT_LOOP
-#define EVDBG(...) log_event(__VA_ARGS__)
+#define EVDBG(...) timed_debug_print(__VA_ARGS__)
 #else
 #define EVDBG(...)
 #endif
@@ -792,12 +792,14 @@ render_prepared_os_window(OSWindow *os_window, unsigned int active_window_id, co
     BorderRects *br = &tab->border_rects;
     draw_borders(br->vao_idx, br->num_border_rects, br->rect_buf, br->is_dirty, os_window->viewport_width, os_window->viewport_height, active_window_bg, num_visible_windows, all_windows_have_same_bg, os_window);
     br->is_dirty = false;
-    if (TD.screen && os_window->num_tabs >= OPT(tab_bar_min_tabs)) draw_cells(TD.vao_idx, &TD, os_window, true, false, NULL);
+    if (TD.screen && os_window->num_tabs >= OPT(tab_bar_min_tabs)) draw_cells(TD.vao_idx, &TD, os_window, true, true, false, NULL);
+    unsigned int num_of_visible_windows = 0;
+    for (unsigned int i = 0; i < tab->num_windows; i++) { if (tab->windows[i].visible) num_of_visible_windows++; }
     for (unsigned int i = 0; i < tab->num_windows; i++) {
         Window *w = tab->windows + i;
         if (w->visible && WD.screen) {
             bool is_active_window = i == tab->active_window;
-            draw_cells(WD.vao_idx, &WD, os_window, is_active_window, true, w);
+            draw_cells(WD.vao_idx, &WD, os_window, is_active_window, false, num_of_visible_windows == 1, w);
             if (WD.screen->start_visual_bell_at != 0) {
                 set_maximum_wait(OPT(repaint_delay));
             }
@@ -819,9 +821,9 @@ no_render_frame_received_recently(OSWindow *w, monotonic_t now, monotonic_t max_
     bool ans = now - w->last_render_frame_received_at > max_wait;
     if (ans && global_state.debug_rendering) {
         if (global_state.is_wayland) {
-            log_error("No render frame received in %.2f seconds", monotonic_t_to_s_double(max_wait));
+            fprintf(stderr, "[%f] No render frame received in %.2f seconds", monotonic_t_to_s_double(now), monotonic_t_to_s_double(max_wait));
         } else  {
-            log_error("No render frame received in %.2f seconds, re-requesting at: %f", monotonic_t_to_s_double(max_wait), monotonic_t_to_s_double(now));
+            fprintf(stderr, "[%f] No render frame received in %.2f seconds, re-requesting", monotonic_t_to_s_double(now), monotonic_t_to_s_double(max_wait));
         }
     }
     return ans;
@@ -1076,14 +1078,10 @@ process_pending_resizes(monotonic_t now) {
 
 static void
 close_os_window(ChildMonitor *self, OSWindow *os_window) {
-    int w = os_window->content_area_width, h = os_window->content_area_height;
+    int w = os_window->window_width, h = os_window->window_height;
     if (os_window->before_fullscreen.is_set && is_os_window_fullscreen(os_window)) {
         w = os_window->before_fullscreen.w; h = os_window->before_fullscreen.h;
     }
-    // On GNOME Wayland w, h are the content area size, we need to add the frame size back
-    int content_area_width, content_area_height;
-    adjust_window_size_for_csd(os_window, w, h, &content_area_width, &content_area_height);
-    w += w - content_area_width; h += h - content_area_height;
     destroy_os_window(os_window);
     call_boss(on_os_window_closed, "Kii", os_window->id, w, h);
     for (size_t t=0; t < os_window->num_tabs; t++) {
