@@ -404,6 +404,29 @@ find_in_memoryview(PyObject *self UNUSED, PyObject *args) {
     return PyLong_FromSsize_t(ans);
 }
 
+#include "terminfo.h"
+
+static PyObject*
+py_terminfo_data(PyObject *self UNUSED, PyObject *args UNUSED) {
+    return PyBytes_FromStringAndSize((const char*)terminfo_data, arraysz(terminfo_data));
+}
+
+static PyObject*
+py_monotonic(PyObject *self UNUSED, PyObject *args UNUSED) {
+    return PyFloat_FromDouble(monotonic_t_to_s_double(monotonic()));
+}
+
+static PyObject*
+py_timed_debug_print(PyObject *self UNUSED, PyObject *args) {
+    const char *payload; Py_ssize_t sz;
+    if (!PyArg_ParseTuple(args, "s#", &payload, &sz)) return NULL;
+    const char *fmt = "%.*s";
+    if (sz && payload[sz-1] == '\n') { fmt = "%.*s\n"; sz--; }
+    timed_debug_print(fmt, sz, payload);
+    Py_RETURN_NONE;
+}
+
+
 static PyMethodDef module_methods[] = {
     METHODB(replace_c0_codes_except_nl_space_tab, METH_O),
     {"wcwidth", (PyCFunction)wcwidth_wrap, METH_O, ""},
@@ -425,6 +448,9 @@ static PyMethodDef module_methods[] = {
     {"shm_open", (PyCFunction)py_shm_open, METH_VARARGS, ""},
     {"shm_unlink", (PyCFunction)py_shm_unlink, METH_VARARGS, ""},
     {"wrapped_kitten_names", (PyCFunction)wrapped_kittens, METH_NOARGS, ""},
+    {"terminfo_data", (PyCFunction)py_terminfo_data, METH_NOARGS, ""},
+    {"monotonic", (PyCFunction)py_monotonic, METH_NOARGS, ""},
+    {"timed_debug_print", (PyCFunction)py_timed_debug_print, METH_VARARGS, ""},
     {"find_in_memoryview", (PyCFunction)find_in_memoryview, METH_VARARGS, ""},
 #ifdef __APPLE__
     METHODB(user_cache_dir, METH_NOARGS),
