@@ -15,6 +15,8 @@
 #define MISSING_GLYPH (NUM_UNDERLINE_STYLES + 2)
 #define MAX_NUM_EXTRA_GLYPHS_PUA 4u
 
+#define debug debug_fonts
+
 static PyObject *python_send_to_gpu_impl = NULL;
 #define current_send_sprite_to_gpu(...) (python_send_to_gpu_impl ? python_send_to_gpu(__VA_ARGS__) : send_sprite_to_gpu(__VA_ARGS__))
 extern PyTypeObject Line_Type;
@@ -430,8 +432,8 @@ has_emoji_presentation(CPUCell *cpu_cell, GPUCell *gpu_cell) {
 }
 
 static bool
-has_cell_text(Font *self, CPUCell *cell) {
-    if (!face_has_codepoint(self->face, cell->ch)) return false;
+has_cell_text(PyObject *face, CPUCell *cell) {
+    if (!face_has_codepoint(face, cell->ch)) return false;
     char_type combining_chars[arraysz(cell->cc_idx)];
     unsigned num_cc = 0;
     for (unsigned i = 0; i < arraysz(cell->cc_idx) && cell->cc_idx[i]; i++) {
@@ -440,29 +442,29 @@ has_cell_text(Font *self, CPUCell *cell) {
     }
     if (num_cc == 0) return true;
     if (num_cc == 1) {
-        if (face_has_codepoint(self->face, combining_chars[0])) return true;
+        if (face_has_codepoint(face, combining_chars[0])) return true;
         char_type ch = 0;
-        if (hb_unicode_compose(hb_unicode_funcs_get_default(), cell->ch, combining_chars[0], &ch) && face_has_codepoint(self->face, ch)) return true;
+        if (hb_unicode_compose(hb_unicode_funcs_get_default(), cell->ch, combining_chars[0], &ch) && face_has_codepoint(face, ch)) return true;
         return false;
     }
     for (unsigned i = 0; i < num_cc; i++) {
-        if (!face_has_codepoint(self->face, combining_chars[i])) return false;
+        if (!face_has_codepoint(face, combining_chars[i])) return false;
     }
     return true;
 }
 
 static void
 output_cell_fallback_data(CPUCell *cell, bool bold, bool italic, bool emoji_presentation, PyObject *face) {
-    printf("U+%x ", cell->ch);
+    debug("U+%x ", cell->ch);
     for (unsigned i = 0; i < arraysz(cell->cc_idx) && cell->cc_idx[i]; i++) {
-        printf("U+%x ", codepoint_for_mark(cell->cc_idx[i]));
+        debug("U+%x ", codepoint_for_mark(cell->cc_idx[i]));
     }
-    if (bold) printf("bold ");
-    if (italic) printf("italic ");
-    if (emoji_presentation) printf("emoji_presentation ");
-    if (PyLong_Check(face)) printf("using previous fallback font at index: ");
-    PyObject_Print(face, stdout, 0);
-    printf("\n");
+    if (bold) debug("bold ");
+    if (italic) debug("italic ");
+    if (emoji_presentation) debug("emoji_presentation ");
+    if (PyLong_Check(face)) debug("using previous fallback font at index: ");
+    PyObject_Print(face, stderr, 0);
+    debug("\n");
 }
 
 PyObject*
@@ -496,16 +498,15 @@ load_fallback_font(FontGroup *fg, CPUCell *cell, bool bold, bool italic, bool em
     Font *af = &fg->fonts[ans];
     if (!init_font(af, face, bold, italic, emoji_presentation)) fatal("Out of memory");
     Py_DECREF(face);
-    if (!has_cell_text(af, cell)) {
+    if (!has_cell_text(af->face, cell)) {
         if (global_state.debug_font_fallback) {
-            printf("The font chosen by the OS for the text: ");
-            printf("U+%x ", cell->ch);
+            debug("The font chosen by the OS for the text: ");
+            debug("U+%x ", cell->ch);
             for (unsigned i = 0; i < arraysz(cell->cc_idx) && cell->cc_idx[i]; i++) {
-                printf("U+%x ", codepoint_for_mark(cell->cc_idx[i]));
+                debug("U+%x ", codepoint_for_mark(cell->cc_idx[i]));
             }
-            printf("is ");
-            PyObject_Print(af->face, stdout, 0);
-            printf(" but it does not actually contain glyphs for that text\n");
+            debug("is "); PyObject_Print(af->face, stderr, 0);
+            debug(" but it does not actually contain glyphs for that text\n");
         }
         del_font(af);
         return MISSING_FONT;
@@ -594,7 +595,7 @@ START_ALLOW_CASE_RANGE
                     ans = fg->bi_font_idx; break;
             }
             if (ans < 0) ans = fg->medium_font_idx;
-            if (!*is_emoji_presentation && has_cell_text(fg->fonts + ans, cpu_cell)) { *is_main_font = true; return ans; }
+            if (!*is_emoji_presentation && has_cell_text((fg->fonts + ans)->face, cpu_cell)) { *is_main_font = true; return ans; }
             return fallback_font(fg, cpu_cell, gpu_cell);
     }
 END_ALLOW_CASE_RANGE
