@@ -144,7 +144,7 @@ def cross(buf: BufType, width: int, height: int, a: int = 1, b: int = 1, c: int
 
 
 def downsample(src: BufType, dest: BufType, dest_width: int, dest_height: int, factor: int = 4) -> None:
-    src_width = 4 * dest_width
+    src_width = factor * dest_width
 
     def average_intensity_in_src(dest_x: int, dest_y: int) -> int:
         src_y = dest_y * factor
@@ -1041,57 +1041,57 @@ box_chars: Dict[str, List[Callable[[BufType, int, int], Any]]] = {
     '▞': [p(quad, x=1), p(quad, y=1)],
     '▟': [p(quad, x=1), p(quad, y=1), p(quad, x=1, y=1)],
 
-    '🬼': [p(smooth_mosaic, a=(0, 0.75), b=(0.5, 1))],
-    '🬽': [p(smooth_mosaic, a=(0, 0.75), b=(1, 1))],
-    '🬾': [p(smooth_mosaic, a=(0, 0.25), b=(0.5, 1))],
-    '🬿': [p(smooth_mosaic, a=(0, 0.25), b=(1, 1))],
+    '🬼': [p(smooth_mosaic, a=(0, 2 / 3), b=(0.5, 1))],
+    '🬽': [p(smooth_mosaic, a=(0, 2 / 3), b=(1, 1))],
+    '🬾': [p(smooth_mosaic, a=(0, 1 / 3), b=(0.5, 1))],
+    '🬿': [p(smooth_mosaic, a=(0, 1 / 3), b=(1, 1))],
     '🭀': [p(smooth_mosaic, a=(0, 0), b=(0.5, 1))],
 
-    '🭁': [p(smooth_mosaic, a=(0, 0.25), b=(0.5, 0))],
-    '🭂': [p(smooth_mosaic, a=(0, 0.25), b=(1, 0))],
-    '🭃': [p(smooth_mosaic, a=(0, 0.75), b=(0.5, 0))],
-    '🭄': [p(smooth_mosaic, a=(0, 0.75), b=(1, 0))],
+    '🭁': [p(smooth_mosaic, a=(0, 1 / 3), b=(0.5, 0))],
+    '🭂': [p(smooth_mosaic, a=(0, 1 / 3), b=(1, 0))],
+    '🭃': [p(smooth_mosaic, a=(0, 2 / 3), b=(0.5, 0))],
+    '🭄': [p(smooth_mosaic, a=(0, 2 / 3), b=(1, 0))],
     '🭅': [p(smooth_mosaic, a=(0, 1), b=(0.5, 0))],
-    '🭆': [p(smooth_mosaic, a=(0, 0.75), b=(1, 0.25))],
+    '🭆': [p(smooth_mosaic, a=(0, 2 / 3), b=(1, 1 / 3))],
 
-    '🭇': [p(smooth_mosaic, a=(0.5, 1), b=(1, 0.75))],
-    '🭈': [p(smooth_mosaic, a=(0, 1), b=(1, 0.75))],
-    '🭉': [p(smooth_mosaic, a=(0.5, 1), b=(1, 0.25))],
-    '🭊': [p(smooth_mosaic, a=(0, 1), b=(1, 0.25))],
+    '🭇': [p(smooth_mosaic, a=(0.5, 1), b=(1, 2 / 3))],
+    '🭈': [p(smooth_mosaic, a=(0, 1), b=(1, 2 / 3))],
+    '🭉': [p(smooth_mosaic, a=(0.5, 1), b=(1, 1 / 3))],
+    '🭊': [p(smooth_mosaic, a=(0, 1), b=(1, 1 / 3))],
     '🭋': [p(smooth_mosaic, a=(0.5, 1), b=(1, 0))],
 
-    '🭌': [p(smooth_mosaic, a=(0.5, 0), b=(1, 0.25))],
-    '🭍': [p(smooth_mosaic, a=(0, 0), b=(1, 0.25))],
-    '🭎': [p(smooth_mosaic, a=(0.5, 0), b=(1, 0.75))],
-    '🭏': [p(smooth_mosaic, a=(0, 0), b=(1, 0.75))],
+    '🭌': [p(smooth_mosaic, a=(0.5, 0), b=(1, 1 / 3))],
+    '🭍': [p(smooth_mosaic, a=(0, 0), b=(1, 1 / 3))],
+    '🭎': [p(smooth_mosaic, a=(0.5, 0), b=(1, 2 / 3))],
+    '🭏': [p(smooth_mosaic, a=(0, 0), b=(1, 2 / 3))],
     '🭐': [p(smooth_mosaic, a=(0.5, 0), b=(1, 1))],
-    '🭑': [p(smooth_mosaic, a=(0, 0.25), b=(1, 0.75))],
+    '🭑': [p(smooth_mosaic, a=(0, 1 / 3), b=(1, 2 / 3))],
 
-    '🭒': [p(smooth_mosaic, lower=False, a=(0, 0.75), b=(0.5, 1))],
-    '🭓': [p(smooth_mosaic, lower=False, a=(0, 0.75), b=(1, 1))],
-    '🭔': [p(smooth_mosaic, lower=False, a=(0, 0.25), b=(0.5, 1))],
-    '🭕': [p(smooth_mosaic, lower=False, a=(0, 0.25), b=(1, 1))],
+    '🭒': [p(smooth_mosaic, lower=False, a=(0, 2 / 3), b=(0.5, 1))],
+    '🭓': [p(smooth_mosaic, lower=False, a=(0, 2 / 3), b=(1, 1))],
+    '🭔': [p(smooth_mosaic, lower=False, a=(0, 1 / 3), b=(0.5, 1))],
+    '🭕': [p(smooth_mosaic, lower=False, a=(0, 1 / 3), b=(1, 1))],
     '🭖': [p(smooth_mosaic, lower=False, a=(0, 0), b=(0.5, 1))],
 
-    '🭗': [p(smooth_mosaic, lower=False, a=(0, 0.25), b=(0.5, 0))],
-    '🭘': [p(smooth_mosaic, lower=False, a=(0, 0.25), b=(1, 0))],
-    '🭙': [p(smooth_mosaic, lower=False, a=(0, 0.75), b=(0.5, 0))],
-    '🭚': [p(smooth_mosaic, lower=False, a=(0, 0.75), b=(1, 0))],
+    '🭗': [p(smooth_mosaic, lower=False, a=(0, 1 / 3), b=(0.5, 0))],
+    '🭘': [p(smooth_mosaic, lower=False, a=(0, 1 / 3), b=(1, 0))],
+    '🭙': [p(smooth_mosaic, lower=False, a=(0, 2 / 3), b=(0.5, 0))],
+    '🭚': [p(smooth_mosaic, lower=False, a=(0, 2 / 3), b=(1, 0))],
     '🭛': [p(smooth_mosaic, lower=False, a=(0, 1), b=(0.5, 0))],
 
-    '🭜': [p(smooth_mosaic, lower=False, a=(0, 0.75), b=(1, 0.25))],
-    '🭝': [p(smooth_mosaic, lower=False, a=(0.5, 1), b=(1, 0.75))],
-    '🭞': [p(smooth_mosaic, lower=False, a=(0, 1), b=(1, 0.75))],
-    '🭟': [p(smooth_mosaic, lower=False, a=(0.5, 1), b=(1, 0.25))],
-    '🭠': [p(smooth_mosaic, lower=False, a=(0, 1), b=(1, 0.25))],
+    '🭜': [p(smooth_mosaic, lower=False, a=(0, 2 / 3), b=(1, 1 / 3))],
+    '🭝': [p(smooth_mosaic, lower=False, a=(0.5, 1), b=(1, 2 / 3))],
+    '🭞': [p(smooth_mosaic, lower=False, a=(0, 1), b=(1, 2 / 3))],
+    '🭟': [p(smooth_mosaic, lower=False, a=(0.5, 1), b=(1, 1 / 3))],
+    '🭠': [p(smooth_mosaic, lower=False, a=(0, 1), b=(1, 1 / 3))],
     '🭡': [p(smooth_mosaic, lower=False, a=(0.5, 1), b=(1, 0))],
 
-    '🭢': [p(smooth_mosaic, lower=False, a=(0.5, 0), b=(1, 0.25))],
-    '🭣': [p(smooth_mosaic, lower=False, a=(0, 0), b=(1, 0.25))],
-    '🭤': [p(smooth_mosaic, lower=False, a=(0.5, 0), b=(1, 0.75))],
-    '🭥': [p(smooth_mosaic, lower=False, a=(0, 0), b=(1, 0.75))],
+    '🭢': [p(smooth_mosaic, lower=False, a=(0.5, 0), b=(1, 1 / 3))],
+    '🭣': [p(smooth_mosaic, lower=False, a=(0, 0), b=(1, 1 / 3))],
+    '🭤': [p(smooth_mosaic, lower=False, a=(0.5, 0), b=(1, 2 / 3))],
+    '🭥': [p(smooth_mosaic, lower=False, a=(0, 0), b=(1, 2 / 3))],
     '🭦': [p(smooth_mosaic, lower=False, a=(0.5, 0), b=(1, 1))],
-    '🭧': [p(smooth_mosaic, lower=False, a=(0, 0.25), b=(1, 0.75))],
+    '🭧': [p(smooth_mosaic, lower=False, a=(0, 1 / 3), b=(1, 2 / 3))],
 
     '🭨': [p(half_triangle, inverted=True)],
     '🭩': [p(half_triangle, which='top', inverted=True)],
