@@ -152,10 +152,16 @@ on_key_input(GLFWkeyevent *ev) {
     const uint32_t key = ev->key, native_key = ev->native_key;
     const char *text = ev->text ? ev->text : "";
 
-    debug("\x1b[33mon_key_input\x1b[m: glfw key: 0x%x native_code: 0x%x action: %s %stext: '%s' state: %d ",
-            key, native_key,
-            (action == GLFW_RELEASE ? "RELEASE" : (action == GLFW_PRESS ? "PRESS" : "REPEAT")),
-            format_mods(mods), text, ev->ime_state);
+    if (OPT(debug_keyboard)) {
+        if (!key && !native_key && text[0]) {
+            debug("\x1b[33mon_IME_input\x1b[m: text: %s ", text);
+        } else {
+            debug("\x1b[33mon_key_input\x1b[m: glfw key: 0x%x native_code: 0x%x action: %s %stext: '%s' state: %d ",
+                    key, native_key,
+                    (action == GLFW_RELEASE ? "RELEASE" : (action == GLFW_PRESS ? "PRESS" : "REPEAT")),
+                    format_mods(mods), text, ev->ime_state);
+        }
+    }
     if (!w) { debug("no active window, ignoring\n"); return; }
     if (OPT(mouse_hide_wait) < 0 && !is_modifier_key(key)) hide_mouse(global_state.callback_os_window);
     Screen *screen = w->render_data.screen;
@@ -165,7 +171,8 @@ on_key_input(GLFWkeyevent *ev) {
         case GLFW_IME_WAYLAND_DONE_EVENT:
             // If we update IME position here it sends GNOME's text input system into
             // an infinite loop. See https://github.com/kovidgoyal/kitty/issues/5105
-            screen_update_overlay_text(screen, NULL);
+            // and also: https://github.com/kovidgoyal/kitty/pull/7283
+            screen_update_overlay_text(screen, text);
             debug("handled wayland IME done event\n");
             return;
         case GLFW_IME_PREEDIT_CHANGED:
@@ -176,7 +183,7 @@ on_key_input(GLFWkeyevent *ev) {
         case GLFW_IME_COMMIT_TEXT:
             if (*text) {
                 schedule_write_to_child(w->id, 1, text, strlen(text));
-                debug("committed pre-edit text: %s\n", text);
+                debug("committed pre-edit text: %s sent to child as text.\n", text);
             } else debug("committed pre-edit text: (null)\n");
             screen_update_overlay_text(screen, NULL);
             return;
