@@ -14,7 +14,7 @@
 extern PyTypeObject Cursor_Type;
 
 static PyObject *
-new(PyTypeObject UNUSED *type, PyObject UNUSED *args, PyObject UNUSED *kwds) {
+new_line_object(PyTypeObject UNUSED *type, PyObject UNUSED *args, PyObject UNUSED *kwds) {
     PyErr_SetString(PyExc_TypeError, "Line objects cannot be instantiated directly, create them using LineBuf.line()");
     return NULL;
 }
@@ -433,10 +433,10 @@ width(Line *self, PyObject *val) {
 }
 
 void
-line_add_combining_char(Line *self, uint32_t ch, unsigned int x) {
-    CPUCell *cell = self->cpu_cells + x;
+line_add_combining_char(CPUCell *cpu_cells, GPUCell *gpu_cells, uint32_t ch, unsigned int x) {
+    CPUCell *cell = cpu_cells + x;
     if (!cell->ch) {
-        if (x > 0 && (self->gpu_cells[x-1].attrs.width) == 2 && self->cpu_cells[x-1].ch) cell = self->cpu_cells + x - 1;
+        if (x > 0 && (gpu_cells[x-1].attrs.width) == 2 && cpu_cells[x-1].ch) cell = cpu_cells + x - 1;
         else return; // don't allow adding combining chars to a null cell
     }
     for (unsigned i = 0; i < arraysz(cell->cc_idx); i++) {
@@ -455,7 +455,7 @@ add_combining_char(Line* self, PyObject *args) {
         PyErr_SetString(PyExc_ValueError, "Column index out of bounds");
         return NULL;
     }
-    line_add_combining_char(self, new_char, x);
+    line_add_combining_char(self->cpu_cells, self->gpu_cells, new_char, x);
     Py_RETURN_NONE;
 }
 
@@ -521,11 +521,10 @@ cursor_from(Line* self, PyObject *args) {
 void
 line_clear_text(Line *self, unsigned int at, unsigned int num, char_type ch) {
     const uint16_t width = ch ? 1 : 0;
-    for (index_type i = at; i < MIN(self->xnum, at + num); i++) {
-        self->cpu_cells[i].ch = ch; memset(self->cpu_cells[i].cc_idx, 0, sizeof(self->cpu_cells[i].cc_idx));
-        self->cpu_cells[i].hyperlink_id = 0;
-        self->gpu_cells[i].attrs.width = width;
-    }
+    const CPUCell cc = {.ch=ch};
+    if (at + num > self->xnum) num = self->xnum > at ? self->xnum - at : 0;
+    memset_array(self->cpu_cells + at, cc, num);
+    for (index_type i = at; i < at + num; i++) self->gpu_cells[i].attrs.width = width;
 }
 
 static PyObject*
@@ -539,25 +538,22 @@ clear_text(Line* self, PyObject *args) {
 }
 
 void
-line_apply_cursor(Line *self, Cursor *cursor, unsigned int at, unsigned int num, bool clear_char) {
-    CellAttrs attrs = cursor_to_attrs(cursor, 0);
-    color_type fg = (cursor->fg & COL_MASK), bg = (cursor->bg & COL_MASK);
-    color_type dfg = cursor->decoration_fg & COL_MASK;
-
-    for (index_type i = at; i < self->xnum && i < at + num; i++) {
-        if (clear_char) {
-            self->cpu_cells[i].ch = BLANK_CHAR;
-            self->cpu_cells[i].hyperlink_id = 0;
-            memset(self->cpu_cells[i].cc_idx, 0, sizeof(self->cpu_cells[i].cc_idx));
-            self->gpu_cells[i].attrs = attrs;
-            clear_sprite_position(self->gpu_cells[i]);
-        } else {
-            attrs.width = self->gpu_cells[i].attrs.width;
-            attrs.mark = self->gpu_cells[i].attrs.mark;
-            self->gpu_cells[i].attrs = attrs;
+line_apply_cursor(Line *self, const Cursor *cursor, unsigned int at, unsigned int num, bool clear_char) {
+    GPUCell gc = cursor_as_gpu_cell(cursor);
+    if (clear_char) {
+#if BLANK_CHAR != 0
+#error This implementation is incorrect for BLANK_CHAR != 0
+#endif
+        if (at + num > self->xnum) { num = at < self->xnum ? self->xnum - at : 0; }
+        memset(self->cpu_cells + at, 0, num * sizeof(CPUCell));
+        memset_array(self->gpu_cells + at, gc, num);
+    } else {
+        for (index_type i = at; i < self->xnum && i < at + num; i++) {
+            gc.attrs.width = self->gpu_cells[i].attrs.width;
+            gc.attrs.mark = self->gpu_cells[i].attrs.mark;
+            gc.sprite_x = self->gpu_cells[i].sprite_x; gc.sprite_y = self->gpu_cells[i].sprite_y; gc.sprite_z = self->gpu_cells[i].sprite_z;
+            memcpy(self->gpu_cells + i, &gc, sizeof(gc));
         }
-        self->gpu_cells[i].fg = fg; self->gpu_cells[i].bg = bg;
-        self->gpu_cells[i].decoration_fg = dfg;
     }
 }
 
@@ -614,7 +610,7 @@ left_shift(Line *self, PyObject *args) {
 }
 
 static color_type
-resolve_color(ColorProfile *cp, color_type val, color_type defval) {
+resolve_color(const ColorProfile *cp, color_type val, color_type defval) {
     switch(val & 0xff) {
         case 1:
             return cp->color_table[(val >> 8) & 0xff];
@@ -626,7 +622,7 @@ resolve_color(ColorProfile *cp, color_type val, color_type defval) {
 }
 
 bool
-colors_for_cell(Line *self, ColorProfile *cp, index_type *x, color_type *fg, color_type *bg, bool *reversed) {
+colors_for_cell(Line *self, const ColorProfile *cp, index_type *x, color_type *fg, color_type *bg, bool *reversed) {
     if (*x >= self->xnum) return false;
     if (*x > 0 && !self->gpu_cells[*x].attrs.width && self->gpu_cells[*x-1].attrs.width == 2) (*x)--;
     *fg = resolve_color(cp, self->gpu_cells[*x].fg, *fg);
@@ -660,6 +656,10 @@ line_set_char(Line *self, unsigned int at, uint32_t ch, unsigned int width, Curs
     }
     self->cpu_cells[at].ch = ch;
     self->cpu_cells[at].hyperlink_id = hyperlink_id;
+    if (OPT(underline_hyperlinks) == UNDERLINE_ALWAYS && hyperlink_id) {
+        g->decoration_fg = ((OPT(url_color) & COL_MASK) << 8) | 2;
+        g->attrs.decoration = OPT(url_style);
+    }
     memset(self->cpu_cells[at].cc_idx, 0, sizeof(self->cpu_cells[at].cc_idx));
 }
 
@@ -714,6 +714,8 @@ decoration_as_sgr(uint8_t decoration) {
         case 1: return "4;";
         case 2: return "4:2;";
         case 3: return "4:3;";
+        case 4: return "4:4";
+        case 5: return "4:5";
         default: return "24;";
     }
 }
@@ -729,8 +731,10 @@ cell_as_sgr(const GPUCell *cell, const GPUCell *prev) {
 #define PA prev->attrs
     bool intensity_differs = CA.bold != PA.bold || CA.dim != PA.dim;
     if (intensity_differs) {
-        if (!CA.bold && !CA.dim) { P("22;"); }
-        else { if (CA.bold) P("1;"); if (CA.dim) P("2;"); }
+        if (CA.bold && CA.dim) { if (!PA.bold) P("1;"); if (!PA.dim) P("2;"); }
+        else {
+            P("22;"); if (CA.bold) P("1;"); if (CA.dim) P("2;");
+        }
     }
     if (CA.italic != PA.italic) P(CA.italic ? "3;" : "23;");
     if (CA.reverse != PA.reverse) P(CA.reverse ? "7;" : "27;");
@@ -951,7 +955,7 @@ PyTypeObject Line_Type = {
     .tp_richcompare = richcmp,
     .tp_doc = "Lines",
     .tp_methods = methods,
-    .tp_new = new
+    .tp_new = new_line_object
 };
 
 Line *alloc_line(void) {
