@@ -1,4 +1,4 @@
-#!/usr/bin/env python3
+#!/usr/bin/env python
 # License: GPL v3 Copyright: 2016, Kovid Goyal <kovid at kovidgoyal.net>
 
 import os
@@ -9,13 +9,12 @@ from functools import partial
 from typing import TYPE_CHECKING, Callable, Generator, Iterator, List, Mapping, Optional, Tuple, Union
 
 from .cli_stub import CLIOptions
-from .constants import kitten_exe
 from .layout.interface import all_layouts
 from .options.types import Options
 from .options.utils import resize_window, to_layout_names, window_size
 from .os_window_size import WindowSize, WindowSizeData, WindowSizes
 from .typing import SpecialWindowInstance
-from .utils import expandvars, log_error, resolve_custom_file, resolved_shell
+from .utils import expandvars, log_error, resolve_custom_file, resolved_shell, shlex_split
 
 if TYPE_CHECKING:
     from .launch import LaunchSpec
@@ -27,7 +26,9 @@ def get_os_window_sizing_data(opts: Options, session: Optional['Session'] = None
         sizes = WindowSizes(WindowSize(*opts.initial_window_width), WindowSize(*opts.initial_window_height))
     else:
         sizes = session.os_window_size
-    return WindowSizeData(sizes, opts.remember_window_size, opts.single_window_margin_width, opts.window_margin_width, opts.window_padding_width)
+    return WindowSizeData(
+        sizes, opts.remember_window_size, opts.single_window_margin_width, opts.window_margin_width,
+        opts.single_window_padding_width, opts.window_padding_width)
 
 
 ResizeSpec = Tuple[str, int]
@@ -61,6 +62,7 @@ class Session:
         self.default_title = default_title
         self.os_window_size: Optional[WindowSizes] = None
         self.os_window_class: Optional[str] = None
+        self.os_window_state: Optional[str] = None
         self.focus_os_window: bool = False
 
     def add_tab(self, opts: Options, name: str = '') -> None:
@@ -81,7 +83,7 @@ class Session:
         needs_expandvars = False
         if isinstance(cmd, str):
             needs_expandvars = True
-            cmd = shlex.split(cmd)
+            cmd = list(shlex_split(cmd))
         spec = parse_launch_args(cmd)
         if needs_expandvars:
             assert isinstance(cmd, list)
@@ -177,6 +179,8 @@ def parse_session(raw: str, opts: Options, environ: Optional[Mapping[str, str]]
                 ans.os_window_size = WindowSizes(WindowSize(*w), WindowSize(*h))
             elif cmd == 'os_window_class':
                 ans.os_window_class = rest
+            elif cmd == 'os_window_state':
+                ans.os_window_state = rest
             elif cmd == 'resize_window':
                 ans.resize_window(rest.split())
             else:
@@ -202,20 +206,23 @@ def create_sessions(
     default_session: Optional[str] = None,
 ) -> Iterator[Session]:
     if args and args.session:
-        environ: Optional[Mapping[str, str]] = None
-        if isinstance(args.session, PreReadSession):
-            session_data = '' + str(args.session)
-            environ = args.session.associated_environ  # type: ignore
+        if args.session == "none":
+            default_session = "none"
         else:
-            if args.session == '-':
-                f = sys.stdin
+            environ: Optional[Mapping[str, str]] = None
+            if isinstance(args.session, PreReadSession):
+                session_data = '' + str(args.session)
+                environ = args.session.associated_environ  # type: ignore
             else:
-                f = open(resolve_custom_file(args.session))
-            with f:
-                session_data = f.read()
-        yield from parse_session(session_data, opts, environ=environ)
-        return
-    if default_session and default_session != 'none':
+                if args.session == '-':
+                    f = sys.stdin
+                else:
+                    f = open(resolve_custom_file(args.session))
+                with f:
+                    session_data = f.read()
+            yield from parse_session(session_data, opts, environ=environ)
+            return
+    if default_session and default_session != 'none' and not getattr(args, 'args', None):
         try:
             with open(default_session) as f:
                 session_data = f.read()
@@ -230,10 +237,8 @@ def create_sessions(
     ans.tabs[-1].layout = current_layout
     if special_window is None:
         cmd = args.args if args and args.args else resolved_shell(opts)
-        if args and args.hold:
-            cmd = [kitten_exe(), '__hold_till_enter__'] + cmd
         from kitty.tabs import SpecialWindow
         cwd: Optional[str] = args.directory if respect_cwd and args else None
-        special_window = SpecialWindow(cmd, cwd_from=cwd_from, cwd=cwd)
+        special_window = SpecialWindow(cmd, cwd_from=cwd_from, cwd=cwd, hold=bool(args and args.hold))
     ans.add_special_window(special_window)
     yield ans
