@@ -4,6 +4,7 @@
 import json
 import os
 import re
+import sys
 import weakref
 from collections import deque
 from contextlib import contextmanager, suppress
@@ -11,7 +12,7 @@ from enum import Enum, IntEnum, auto
 from functools import lru_cache, partial
 from gettext import gettext as _
 from itertools import chain
-from time import monotonic, time_ns
+from time import time_ns
 from typing import (
     TYPE_CHECKING,
     Any,
@@ -71,6 +72,7 @@ from .fast_data_types import (
     is_css_pointer_name_valid,
     last_focused_os_window_id,
     mark_os_window_dirty,
+    monotonic,
     mouse_selection,
     move_cursor_to_mouse_if_in_prompt,
     pointer_name_to_css_name,
@@ -553,7 +555,7 @@ class Window:
         self.actions_on_removal: List[Callable[['Window'], None]] = []
         self.current_marker_spec: Optional[Tuple[str, Union[str, Tuple[Tuple[int, str], ...]]]] = None
         self.kitten_result_processors: List[Callable[['Window', Any], None]] = []
-        self.pty_resized_once = False
+        self.child_is_launched = False
         self.last_reported_pty_size = (-1, -1, -1, -1)
         self.needs_attention = False
         self.ignore_focus_changes = self.initial_ignore_focus_changes
@@ -833,12 +835,18 @@ class Window:
             max(0, new_geometry.right - new_geometry.left), max(0, new_geometry.bottom - new_geometry.top))
         update_ime_position = False
         if current_pty_size != self.last_reported_pty_size:
-            get_boss().child_monitor.resize_pty(self.id, *current_pty_size)
+            boss = get_boss()
+            boss.child_monitor.resize_pty(self.id, *current_pty_size)
             self.last_resized_at = monotonic()
-            if not self.pty_resized_once:
-                self.pty_resized_once = True
+            if not self.child_is_launched:
                 self.child.mark_terminal_ready()
+                self.child_is_launched = True
                 update_ime_position = True
+                if boss.args.debug_rendering:
+                    now = monotonic()
+                    print(f'[{now:.3f}] Child launched', file=sys.stderr)
+            elif boss.args.debug_rendering:
+                print(f'[{monotonic():.3f}] SIGWINCH sent to child in window: {self.id} with size: {current_pty_size}', file=sys.stderr)
             self.last_reported_pty_size = current_pty_size
         else:
             mark_os_window_dirty(self.os_window_id)
@@ -1764,28 +1772,28 @@ class Window:
     def clear_selection(self) -> None:
         self.screen.clear_selection()
 
-    @ac('sc', 'Scroll up by one line when in main screen')
+    @ac('sc', 'Scroll up by one line when in main screen. To scroll by different amounts, you can map the remote_control scroll-window action.')
     def scroll_line_up(self) -> Optional[bool]:
         if self.screen.is_main_linebuf():
             self.screen.scroll(SCROLL_LINE, True)
             return None
         return True
 
-    @ac('sc', 'Scroll down by one line when in main screen')
+    @ac('sc', 'Scroll down by one line when in main screen. To scroll by different amounts, you can map the remote_control scroll-window action.')
     def scroll_line_down(self) -> Optional[bool]:
         if self.screen.is_main_linebuf():
             self.screen.scroll(SCROLL_LINE, False)
             return None
         return True
 
-    @ac('sc', 'Scroll up by one page when in main screen')
+    @ac('sc', 'Scroll up by one page when in main screen. To scroll by different amounts, you can map the remote_control scroll-window action.')
     def scroll_page_up(self) -> Optional[bool]:
         if self.screen.is_main_linebuf():
             self.screen.scroll(SCROLL_PAGE, True)
             return None
         return True
 
-    @ac('sc', 'Scroll down by one page when in main screen')
+    @ac('sc', 'Scroll down by one page when in main screen. To scroll by different amounts, you can map the remote_control scroll-window action.')
     def scroll_page_down(self) -> Optional[bool]:
         if self.screen.is_main_linebuf():
             self.screen.scroll(SCROLL_PAGE, False)
