@@ -41,7 +41,7 @@ func read_relevant_kitty_opts(path string) KittyOpts {
 		return nil
 	}
 	cp := config.ConfigParser{LineHandler: handle_line}
-	cp.ParseFiles(path)
+	_ = cp.ParseFiles(path)
 	if ans.Shell == "" {
 		ans.Shell = kitty.KittyConfigDefaults.Shell
 	}
@@ -131,10 +131,7 @@ func get_shell_name(argv0 string) (ans string) {
 	if strings.HasSuffix(strings.ToLower(ans), ".exe") {
 		ans = ans[:len(ans)-4]
 	}
-	if strings.HasPrefix(ans, "-") {
-		ans = ans[1:]
-	}
-	return
+	return strings.TrimPrefix(ans, "-")
 }
 
 func rc_modification_allowed(ksi string) bool {
@@ -147,7 +144,7 @@ func rc_modification_allowed(ksi string) bool {
 	return ksi != ""
 }
 
-func RunShell(shell_cmd []string, shell_integration_env_var_val string) (err error) {
+func RunShell(shell_cmd []string, shell_integration_env_var_val, cwd string) (err error) {
 	shell_name := get_shell_name(shell_cmd[0])
 	var shell_env map[string]string
 	if rc_modification_allowed(shell_integration_env_var_val) && shell_integration.IsSupportedShell(shell_name) {
@@ -166,7 +163,7 @@ func RunShell(shell_cmd []string, shell_integration_env_var_val string) (err err
 		shell_env = env
 	}
 	exe := shell_cmd[0]
-	if runtime.GOOS == "darwin" {
+	if runtime.GOOS == "darwin" && (os.Getenv("KITTY_RUNNING_SHELL_INTEGRATION_TEST") != "1" || os.Getenv("KITTY_RUNNING_BASH_INTEGRATION_TEST") != "") {
 		// ensure shell runs in login mode. On macOS lots of people use ~/.bash_profile instead of ~/.bashrc
 		// which means they expect the shell to run in login mode always. Le Sigh.
 		shell_cmd[0] = "-" + filepath.Base(shell_cmd[0])
@@ -181,9 +178,15 @@ func RunShell(shell_cmd []string, shell_integration_env_var_val string) (err err
 		env = os.Environ()
 	}
 	// fmt.Println(fmt.Sprintf("%s %v\n%#v", utils.FindExe(exe), shell_cmd, env))
+	if cwd != "" {
+		_ = os.Chdir(cwd)
+	}
 	return unix.Exec(utils.FindExe(exe), shell_cmd, env)
 }
 
+var debugprintln = tty.DebugPrintln
+var _ = debugprintln
+
 func RunCommandRestoringTerminalToSaneStateAfter(cmd []string) {
 	exe := utils.FindExe(cmd[0])
 	c := exec.Command(exe, cmd[1:]...)
@@ -194,16 +197,19 @@ func RunCommandRestoringTerminalToSaneStateAfter(cmd []string) {
 	if err == nil {
 		var state_before unix.Termios
 		if term.Tcgetattr(&state_before) == nil {
-			term.WriteString(loop.SAVE_PRIVATE_MODE_VALUES)
+			if _, err = term.WriteString(loop.SAVE_PRIVATE_MODE_VALUES); err != nil {
+				fmt.Fprintln(os.Stderr, "failed to write to controlling terminal with error:", err)
+				return
+			}
 			defer func() {
-				term.WriteString(strings.Join([]string{
+				_, _ = term.WriteString(strings.Join([]string{
 					loop.RESTORE_PRIVATE_MODE_VALUES,
 					"\x1b[=u",                      // reset kitty keyboard protocol to legacy
 					"\x1b[1 q",                     // blinking block cursor
 					loop.DECTCEM.EscapeCodeToSet(), // cursor visible
 					"\x1b]112\a",                   // reset cursor color
 				}, ""))
-				term.Tcsetattr(tty.TCSANOW, &state_before)
+				_ = term.Tcsetattr(tty.TCSANOW, &state_before)
 				term.Close()
 			}()
 		} else {
